@@ -1,3 +1,4 @@
+from random import Random
 from eth2spec.utils import bls
 
 from eth2spec.test.helpers.state import (
@@ -9,7 +10,7 @@ from eth2spec.test.helpers.block import (
     sign_block,
     transition_unsigned_block,
 )
-from eth2spec.test.helpers.keys import privkeys, pubkeys
+from eth2spec.test.helpers.keys import pubkeys
 from eth2spec.test.helpers.attester_slashings import (
     get_valid_attester_slashing_by_indices,
     get_valid_attester_slashing,
@@ -18,11 +19,22 @@ from eth2spec.test.helpers.attester_slashings import (
 from eth2spec.test.helpers.proposer_slashings import get_valid_proposer_slashing, check_proposer_slashing_effect
 from eth2spec.test.helpers.attestations import get_valid_attestation
 from eth2spec.test.helpers.deposits import prepare_state_and_deposit
+from eth2spec.test.helpers.voluntary_exits import prepare_signed_exits
 from eth2spec.test.helpers.shard_transitions import get_shard_transition_of_committee
+from eth2spec.test.helpers.multi_operations import (
+    run_slash_and_exit,
+    run_test_full_random_operations,
+)
 
 from eth2spec.test.context import (
-    PHASE0, PHASE1,
-    spec_state_test, with_all_phases, expect_assertion_error, always_bls, with_phases,
+    PHASE0, PHASE1, MINIMAL,
+    spec_test, spec_state_test, dump_skipping_message,
+    with_phases, with_all_phases, single_phase,
+    expect_assertion_error, always_bls,
+    disable_process_reveal_deadlines,
+    with_configs,
+    with_custom_state,
+    large_validator_set,
 )
 
 
@@ -70,6 +82,7 @@ def test_same_slot_block_transition(spec, state):
 def test_empty_block_transition(spec, state):
     pre_slot = state.slot
     pre_eth1_votes = len(state.eth1_data_votes)
+    pre_mix = spec.get_randao_mix(state, spec.get_current_epoch(state))
 
     yield 'pre', state
 
@@ -82,7 +95,32 @@ def test_empty_block_transition(spec, state):
 
     assert len(state.eth1_data_votes) == pre_eth1_votes + 1
     assert spec.get_block_root_at_slot(state, pre_slot) == signed_block.message.parent_root
-    assert spec.get_randao_mix(state, spec.get_current_epoch(state)) != spec.Bytes32()
+    assert spec.get_randao_mix(state, spec.get_current_epoch(state)) != pre_mix
+
+
+@with_all_phases
+@with_configs([MINIMAL],
+              reason="mainnet config leads to larger validator set than limit of public/private keys pre-generated")
+@spec_test
+@with_custom_state(balances_fn=large_validator_set, threshold_fn=lambda spec: spec.EJECTION_BALANCE)
+@single_phase
+def test_empty_block_transition_large_validator_set(spec, state):
+    pre_slot = state.slot
+    pre_eth1_votes = len(state.eth1_data_votes)
+    pre_mix = spec.get_randao_mix(state, spec.get_current_epoch(state))
+
+    yield 'pre', state
+
+    block = build_empty_block_for_next_slot(spec, state)
+
+    signed_block = state_transition_and_sign_block(spec, state, block)
+
+    yield 'blocks', [signed_block]
+    yield 'post', state
+
+    assert len(state.eth1_data_votes) == pre_eth1_votes + 1
+    assert spec.get_block_root_at_slot(state, pre_slot) == signed_block.message.parent_root
+    assert spec.get_randao_mix(state, spec.get_current_epoch(state)) != pre_mix
 
 
 def process_and_sign_block_without_header_validations(spec, state, block):
@@ -288,13 +326,34 @@ def test_empty_epoch_transition(spec, state):
         assert spec.get_block_root_at_slot(state, slot) == block.parent_root
 
 
+@with_all_phases
+@with_configs([MINIMAL],
+              reason="mainnet config leads to larger validator set than limit of public/private keys pre-generated")
+@spec_test
+@with_custom_state(balances_fn=large_validator_set, threshold_fn=lambda spec: spec.EJECTION_BALANCE)
+@single_phase
+def test_empty_epoch_transition_large_validator_set(spec, state):
+    pre_slot = state.slot
+    yield 'pre', state
+
+    block = build_empty_block(spec, state, state.slot + spec.SLOTS_PER_EPOCH)
+
+    signed_block = state_transition_and_sign_block(spec, state, block)
+
+    yield 'blocks', [signed_block]
+    yield 'post', state
+
+    assert state.slot == block.slot
+    for slot in range(pre_slot, state.slot):
+        assert spec.get_block_root_at_slot(state, slot) == block.parent_root
+
+
 @with_all_phases
 @spec_state_test
 def test_empty_epoch_transition_not_finalizing(spec, state):
-    # Don't run for non-minimal configs, it takes very long, and the effect
-    # of calling finalization/justification is just the same as with the minimal configuration.
     if spec.SLOTS_PER_EPOCH > 8:
-        return
+        return dump_skipping_message("Skip mainnet config for saving time."
+                                     " Minimal config suffice to cover the target-of-test.")
 
     # copy for later balance lookups.
     pre_balances = list(state.balances)
@@ -480,9 +539,8 @@ def test_attester_slashing(spec, state):
 @with_all_phases
 @spec_state_test
 def test_duplicate_attester_slashing(spec, state):
-    # Skip test if config cannot handle multiple AttesterSlashings per block
     if spec.MAX_ATTESTER_SLASHINGS < 2:
-        return
+        return dump_skipping_message("Skip test if config cannot handle multiple AttesterSlashings per block")
 
     attester_slashing = get_valid_attester_slashing(spec, state, signed_1=True, signed_2=True)
     attester_slashings = [attester_slashing, attester_slashing.copy()]
@@ -506,12 +564,11 @@ def test_duplicate_attester_slashing(spec, state):
 
 # All AttesterSlashing tests should be adopted for Phase 1 but helper support is not yet there
 
-@with_phases([PHASE0])
+@with_all_phases
 @spec_state_test
 def test_multiple_attester_slashings_no_overlap(spec, state):
-    # Skip test if config cannot handle multiple AttesterSlashings per block
     if spec.MAX_ATTESTER_SLASHINGS < 2:
-        return
+        return dump_skipping_message("Skip test if config cannot handle multiple AttesterSlashings per block")
 
     # copy for later balance lookups.
     pre_state = state.copy()
@@ -547,12 +604,11 @@ def test_multiple_attester_slashings_no_overlap(spec, state):
     check_attester_slashing_effect(spec, pre_state, state, full_indices)
 
 
-@with_phases([PHASE0])
+@with_all_phases
 @spec_state_test
 def test_multiple_attester_slashings_partial_overlap(spec, state):
-    # Skip test if config cannot handle multiple AttesterSlashings per block
     if spec.MAX_ATTESTER_SLASHINGS < 2:
-        return
+        return dump_skipping_message("Skip test if config cannot handle multiple AttesterSlashings per block")
 
     # copy for later balance lookups.
     pre_state = state.copy()
@@ -744,26 +800,13 @@ def test_attestation(spec, state):
     assert spec.hash_tree_root(state.previous_epoch_attestations) == pre_current_attestations_root
 
 
-def prepare_signed_exits(spec, state, indices):
-    domain = spec.get_domain(state, spec.DOMAIN_VOLUNTARY_EXIT)
-
-    def create_signed_exit(index):
-        exit = spec.VoluntaryExit(
-            epoch=spec.get_current_epoch(state),
-            validator_index=index,
-        )
-        signing_root = spec.compute_signing_root(exit, domain)
-        return spec.SignedVoluntaryExit(message=exit, signature=bls.Sign(privkeys[index], signing_root))
-
-    return [create_signed_exit(index) for index in indices]
-
-
 # In phase1 a committee is computed for SHARD_COMMITTEE_PERIOD slots ago,
 # exceeding the minimal-config randao mixes memory size.
 # Applies to all voluntary-exit sanity block tests.
 
-@with_phases([PHASE0])
+@with_all_phases
 @spec_state_test
+@disable_process_reveal_deadlines
 def test_voluntary_exit(spec, state):
     validator_index = spec.get_active_validator_indices(state, spec.get_current_epoch(state))[-1]
 
@@ -790,7 +833,7 @@ def test_voluntary_exit(spec, state):
     assert state.validators[validator_index].exit_epoch < spec.FAR_FUTURE_EPOCH
 
 
-@with_phases([PHASE0])
+@with_all_phases
 @spec_state_test
 def test_double_validator_exit_same_block(spec, state):
     validator_index = spec.get_active_validator_indices(state, spec.get_current_epoch(state))[-1]
@@ -811,8 +854,9 @@ def test_double_validator_exit_same_block(spec, state):
     yield 'post', None
 
 
-@with_phases([PHASE0])
+@with_all_phases
 @spec_state_test
+@disable_process_reveal_deadlines
 def test_multiple_different_validator_exits_same_block(spec, state):
     validator_indices = [
         spec.get_active_validator_indices(state, spec.get_current_epoch(state))[i]
@@ -843,6 +887,23 @@ def test_multiple_different_validator_exits_same_block(spec, state):
         assert state.validators[index].exit_epoch < spec.FAR_FUTURE_EPOCH
 
 
+@with_all_phases
+@spec_state_test
+@disable_process_reveal_deadlines
+def test_slash_and_exit_same_index(spec, state):
+    validator_index = spec.get_active_validator_indices(state, spec.get_current_epoch(state))[-1]
+    yield from run_slash_and_exit(spec, state, validator_index, validator_index, valid=False)
+
+
+@with_all_phases
+@spec_state_test
+@disable_process_reveal_deadlines
+def test_slash_and_exit_diff_index(spec, state):
+    slash_index = spec.get_active_validator_indices(state, spec.get_current_epoch(state))[-1]
+    exit_index = spec.get_active_validator_indices(state, spec.get_current_epoch(state))[-2]
+    yield from run_slash_and_exit(spec, state, slash_index, exit_index)
+
+
 @with_all_phases
 @spec_state_test
 def test_balance_driven_status_transitions(spec, state):
@@ -888,9 +949,9 @@ def test_historical_batch(spec, state):
 @with_all_phases
 @spec_state_test
 def test_eth1_data_votes_consensus(spec, state):
-    # Don't run when it will take very, very long to simulate. Minimal configuration suffices.
     if spec.EPOCHS_PER_ETH1_VOTING_PERIOD > 2:
-        return
+        return dump_skipping_message("Skip test if config with longer `EPOCHS_PER_ETH1_VOTING_PERIOD` for saving time."
+                                     " Minimal config suffice to cover the target-of-test.")
 
     voting_period_slots = spec.EPOCHS_PER_ETH1_VOTING_PERIOD * spec.SLOTS_PER_EPOCH
 
@@ -932,9 +993,9 @@ def test_eth1_data_votes_consensus(spec, state):
 @with_all_phases
 @spec_state_test
 def test_eth1_data_votes_no_consensus(spec, state):
-    # Don't run when it will take very, very long to simulate. Minimal configuration suffices.
     if spec.EPOCHS_PER_ETH1_VOTING_PERIOD > 2:
-        return
+        return dump_skipping_message("Skip test if config with longer `EPOCHS_PER_ETH1_VOTING_PERIOD` for saving time."
+                                     " Minimal config suffice to cover the target-of-test.")
 
     voting_period_slots = spec.EPOCHS_PER_ETH1_VOTING_PERIOD * spec.SLOTS_PER_EPOCH
 
@@ -961,3 +1022,27 @@ def test_eth1_data_votes_no_consensus(spec, state):
 
     yield 'blocks', blocks
     yield 'post', state
+
+
+@with_phases([PHASE0])
+@spec_state_test
+def test_full_random_operations_0(spec, state):
+    yield from run_test_full_random_operations(spec, state, rng=Random(2020))
+
+
+@with_phases([PHASE0])
+@spec_state_test
+def test_full_random_operations_1(spec, state):
+    yield from run_test_full_random_operations(spec, state, rng=Random(2021))
+
+
+@with_phases([PHASE0])
+@spec_state_test
+def test_full_random_operations_2(spec, state):
+    yield from run_test_full_random_operations(spec, state, rng=Random(2022))
+
+
+@with_phases([PHASE0])
+@spec_state_test
+def test_full_random_operations_3(spec, state):
+    yield from run_test_full_random_operations(spec, state, rng=Random(2023))
